% Gmres: Verfahren der generalisierten Residuenminimierung
% 
% Anwendungsbeispiele: Loese A*x=b;
%
% x =  Gmres(A,b);
% x =  Gmres(A,b,tol);
% x =  Gmres(A,b,tol,maxit);
% x =  Gmres(A,b,tol,maxit,x0);
%
function x = gmres5(A,b,tol,maxit,x0)
% 
% OUTPUT VARIABLEN:
% -----------------
% x: Loesungsvector 
% 
% INPUT VARIABLEN:
% ----------------
% A: quadratische n * n Matrix.
% b: rechte Seite.
% tol: [OPTIONAL] Toleranz. Default = 1e-6
% maxit: [OPTIONAL] Maximale Anzahl von Iterationen. Default = min(n,30)
% x0: [OPTIONAL] Startvector fuer Iteration. Default =  Nullvector

% CHECK THE INPUT ARGUMENTS
  if (nargin < 2)
    error('Funktion braucht mehr Parameter.');
  else
    [m,n] = size(A);
    if (m ~= n)
      error('Matrix ist nicht quadratisch.');
    end
    if ~isequal(size(b),[m,1])
      error('Rechte Seite hat nicht die richtige Dimension.');
    end
  end
  if (nargin < 3) | isempty(tol)
    tol = 1.0e-6;
  end
  if (nargin < 4) | isempty(maxit)
    maxit = min(n,30);
  end
  x0 = zeros(n,1);
  if (nargin < 5) | isempty(x0)
    x = zeros(n,1);
  else
    if ~isequal(size(x0),[n,1])
      error('Startvector x0 hat nicht die richtige Dimension.');
    end
    x = x0;
  end

% CHECK FOR TRIVIAL SOLUTION
  normb = norm(b);
  if (normb == 0)
    x = zeros(n,1);
    return
  end
%  
% MAIN ALGORITHM
%
  tolb = tol * normb;
  r = b - A * x;      
  normr = norm(r);
  Gamma=normr*eye(maxit+1,1);
% Hessenberg Matrix
  H = zeros(maxit+1);
% orthogonale Basis 
  V = zeros(n,maxit+1);
% Givensparameter
  c=zeros(maxit+1,1);
  s=zeros(maxit+1,1);
% Dimension des Krylovraums
  kdim = 0;

  V(:,1) = r/normr;
  % iterate
  for j = 1:maxit
    if (normr <= tolb)               
%     converged
%     berechne     
      break
    else
      kdim = kdim+1;
    end
    w = A*V(:,j);
%   Modified Gram-Schmidt
    for i =1:j
      H(i,j) = w'*V(:,i);
      w = w - H(i,j)*V(:,i);
    end
    H(j+1,j) = norm(w);
%   transformiere j-te Spalte der Hessenbergmatrix mit akkumulierten 
%   Givensrotationen (H wird mit R ueberschrieben)
    for i = 1:j-1
      H(i:i+1,j) = [c(i),s(i);-s(i),c(i)] * H(i:i+1,j);
    end
    beta=norm(H(j:j+1,j));
    if(beta ~= 0)
         V(:,j+1)=w/H(j+1,j);
    else
%      'happy breakdown' 
    end
    if beta~=0
%       Berechne neue Givensrotation
        s(j)=H(j+1,j)/beta;
        c(j)=H(j,j)/beta;
%       Anwendung auf  H(j:j+1,j)
	H(j,j) = beta; 
        H(j+1,j)=0;
%       Anwendung auf  b(j:j+1,j)
	Gamma(j+1) = -s(j)*Gamma(j); 
	Gamma(j) = c(j)*Gamma(j);
    end
    normr = abs(Gamma(j+1));
  end
% Berechne die Basiskoeffizienten, loese H*Alpha = Gamma
  Alpha = H(1:kdim,1:kdim)\Gamma(1:kdim);
  x = x + V(:,1:kdim) * Alpha;
  r = b - A*x;
  normr = norm(r);
  if (normr > tolb)               
     warning('Verfahren hat nicht konvergiert')
  end
  return
